@@ -750,14 +750,16 @@ static AOM_INLINE void mode_estimation(AV1_COMP *cpi,
 
       best_inter_cost = inter_cost;
       best_mv[0].as_int = best_rfidx_mv.as_int;
-      if (best_inter_cost < best_intra_cost) {
-        best_mode = NEWMV;
-        xd->mi[0]->ref_frame[0] = best_rf_idx + LAST_FRAME;
-        xd->mi[0]->mv[0].as_int = best_mv[0].as_int;
-      }
     }
   }
 
+  if (best_rf_idx != -1 && best_inter_cost < best_intra_cost) {
+    best_mode = NEWMV;
+    xd->mi[0]->ref_frame[0] = best_rf_idx + LAST_FRAME;
+    xd->mi[0]->mv[0].as_int = best_mv[0].as_int;
+  }
+
+  // Start compound predition search.
   int comp_ref_frames[3][2] = {
     { 0, 4 },
     { 0, 6 },
@@ -816,8 +818,14 @@ static AOM_INLINE void mode_estimation(AV1_COMP *cpi,
       tpl_data->src_ref_frame[rf_idx1],
     };
 
-    xd->mi[0]->ref_frame[0] = LAST_FRAME;
-    xd->mi[0]->ref_frame[1] = ALTREF_FRAME;
+    xd->mi[0]->ref_frame[0] = rf_idx0 + LAST_FRAME;
+    xd->mi[0]->ref_frame[1] = rf_idx1 + LAST_FRAME;
+    xd->mi[0]->mode = NEW_NEWMV;
+    const int8_t ref_frame_type = av1_ref_frame_type(xd->mi[0]->ref_frame);
+    // Set up ref_mv for av1_joint_motion_search().
+    CANDIDATE_MV *this_ref_mv_stack = x->mbmi_ext.ref_mv_stack[ref_frame_type];
+    this_ref_mv_stack[xd->mi[0]->ref_mv_idx].this_mv = single_mv[rf_idx0];
+    this_ref_mv_stack[xd->mi[0]->ref_mv_idx].comp_mv = single_mv[rf_idx1];
 
     struct buf_2d yv12_mb[2][MAX_MB_PLANE];
     for (int i = 0; i < 2; ++i) {
@@ -859,15 +867,17 @@ static AOM_INLINE void mode_estimation(AV1_COMP *cpi,
       best_inter_cost = inter_cost;
       best_mv[0] = tmp_mv[0];
       best_mv[1] = tmp_mv[1];
-
-      if (best_inter_cost < best_intra_cost) {
-        best_mode = NEW_NEWMV;
-        xd->mi[0]->ref_frame[0] = rf_idx0 + LAST_FRAME;
-        xd->mi[0]->ref_frame[1] = rf_idx1 + LAST_FRAME;
-      }
     }
   }
 
+  if (best_cmp_rf_idx != -1 && best_inter_cost < best_intra_cost) {
+    best_mode = NEW_NEWMV;
+    const int best_rf_idx0 = comp_ref_frames[best_cmp_rf_idx][0];
+    const int best_rf_idx1 = comp_ref_frames[best_cmp_rf_idx][1];
+    xd->mi[0]->ref_frame[0] = best_rf_idx0 + LAST_FRAME;
+    xd->mi[0]->ref_frame[1] = best_rf_idx1 + LAST_FRAME;
+  }
+
   if (best_inter_cost < INT64_MAX) {
     xd->mi[0]->mv[0].as_int = best_mv[0].as_int;
     xd->mi[0]->mv[1].as_int = best_mv[1].as_int;
@@ -1270,6 +1280,9 @@ static AOM_INLINE void init_mc_flow_dispenser(AV1_COMP *cpi, int frame_idx,
                            6;
 
   av1_init_tpl_txfm_stats(tpl_txfm_stats);
+
+  // Initialize x->mbmi_ext when compound predictions are enabled.
+  if (cpi->sf.tpl_sf.allow_compound_pred) av1_zero(x->mbmi_ext);
 }
 
 // This function stores the motion estimation dependencies of all the blocks in
@@ -1440,10 +1453,7 @@ static AOM_INLINE void init_gop_frames_for_tpl(
     }
     const int true_disp = (int)(tpl_frame->frame_display_index);
 
-    av1_get_ref_frames(ref_frame_map_pairs, true_disp,
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
-                       cpi, gf_index, 0,
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
+    av1_get_ref_frames(ref_frame_map_pairs, true_disp, cpi, gf_index, 0,
                        remapped_ref_idx);
 
     int refresh_mask =
@@ -1528,10 +1538,7 @@ static AOM_INLINE void init_gop_frames_for_tpl(
 #endif  // CONFIG_BITRATE_ACCURACY && CONFIG_THREE_PASS
     gf_group->q_val[gf_index] = *pframe_qindex;
     const int true_disp = (int)(tpl_frame->frame_display_index);
-    av1_get_ref_frames(ref_frame_map_pairs, true_disp,
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
-                       cpi, gf_index, 0,
-#endif
+    av1_get_ref_frames(ref_frame_map_pairs, true_disp, cpi, gf_index, 0,
                        remapped_ref_idx);
     int refresh_mask =
         av1_get_refresh_frame_flags(cpi, &frame_params, frame_update_type,
